@@ -1,5 +1,5 @@
 """Override rich's markdown renderer with custom components."""
-from __future__ import annotations
+
 
 import dataclasses
 import io
@@ -8,7 +8,7 @@ import pathlib
 import textwrap
 from io import BytesIO
 from pathlib import Path
-from typing import Iterable, Iterator, Literal, Optional, Union
+from typing import Iterable, Iterator, Optional, Union
 from urllib import parse
 
 import httpx
@@ -24,6 +24,7 @@ from rich.table import Table
 from rich.text import Text
 
 from nbpreview.component.content.output.result import drawing, link, table
+from nbpreview.component.content.output.result.drawing import ImageDrawing
 from nbpreview.component.content.output.result.table import TableSection
 
 
@@ -162,12 +163,13 @@ class CustomImageItem(markdown.ImageItem):
     nerd_font: bool = False
     unicode: bool = True
     images: bool = True
-    image_drawing: Literal["block", "character", "braille"] = "block"
+    image_drawing: ImageDrawing = "block"
     color: bool = True
     characters: Optional[str] = None
     files: bool = True
     hide_hyperlink_hints: bool = False
     negative_space: bool = True
+    relative_dir: Path = dataclasses.field(default_factory=pathlib.Path)
 
     def __init__(self, destination: str, hyperlinks: bool) -> None:
         """Constructor."""
@@ -178,8 +180,13 @@ class CustomImageItem(markdown.ImageItem):
             # destination comes in a url quoted format, which will turn
             # Windows-like paths into %5c, unquote here to that pathlib
             # understands correctly
-            unquoted_destination = parse.unquote(self.destination)
-            self.path = pathlib.Path(unquoted_destination).resolve()
+            if (
+                destination_path := pathlib.Path(parse.unquote(self.destination))
+            ).is_absolute():
+                self.path = destination_path
+            else:
+                self.path = self.relative_dir / destination_path
+            self.path = self.path.resolve()
             self.destination = os.fsdecode(self.path)
             content = self.path
             self.is_url = False
@@ -188,7 +195,7 @@ class CustomImageItem(markdown.ImageItem):
             self.path = pathlib.Path(yarl.URL(self.destination).path)
             content = _get_url_content(self.destination)
         self.extension = self.path.suffix.lstrip(".")
-        if content is not None:
+        if content is not None and (self.images or (self.is_url and self.files)):
             try:
                 with Image.open(content) as image:
                     with io.BytesIO() as output:
@@ -205,7 +212,7 @@ class CustomImageItem(markdown.ImageItem):
         else:
             self.image_data = None
 
-        super().__init__(destination=destination, hyperlinks=hyperlinks)
+        super().__init__(destination=self.destination, hyperlinks=hyperlinks)
 
     def __rich_console__(
         self, console: Console, options: ConsoleOptions
@@ -223,32 +230,35 @@ class CustomImageItem(markdown.ImageItem):
                 hyperlinks=self.hyperlinks,
                 hide_hyperlink_hints=self.hide_hyperlink_hints,
             )
+
         else:
-            rendered_link = link.ImageLink(
-                content=self.image_data,
-                file_extension=self.extension,
-                unicode=self.unicode,
-                hyperlinks=self.hyperlinks,
+            rendered_link = link.Link(
+                path=f"file://{self.destination}",
                 nerd_font=self.nerd_font,
-                files=self.files,
-                hide_hyperlink_hints=self.hide_hyperlink_hints,
+                unicode=self.unicode,
                 subject=title,
+                emoji_name="framed_picture",
+                nerd_font_icon="",
+                hyperlinks=self.hyperlinks,
+                hide_hyperlink_hints=self.hide_hyperlink_hints,
             )
+
         yield rendered_link
 
-        fallback_title = self.destination.strip("/").rsplit("/", 1)[-1]
-        rendered_drawing = drawing.choose_drawing(
-            image=self.image_data,
-            fallback_text=self.text.plain or fallback_title,
-            image_type=f"image/{self.extension}",
-            image_drawing=self.image_drawing,
-            color=self.color,
-            negative_space=self.negative_space,
-            characters=self.characters,
-        )
-        if rendered_drawing is not None:
-            yield text.Text("")
-            yield rendered_drawing
+        if self.images:
+            fallback_title = self.destination.strip("/").rsplit("/", 1)[-1]
+            rendered_drawing = drawing.choose_drawing(
+                image=self.image_data,
+                fallback_text=self.text.plain or fallback_title,
+                image_type=f"image/{self.extension}",
+                image_drawing=self.image_drawing,
+                color=self.color,
+                negative_space=self.negative_space,
+                characters=self.characters,
+            )
+            if rendered_drawing is not None:
+                yield text.Text("")
+                yield rendered_drawing
 
 
 class MarkdownOverwrite(markdown.Markdown):
@@ -266,14 +276,16 @@ class MarkdownOverwrite(markdown.Markdown):
         nerd_font: bool = False,
         unicode: bool = True,
         images: bool = True,
-        image_drawing: Literal["block", "character", "braille"] = "block",
+        image_drawing: ImageDrawing = "block",
         color: bool = True,
         negative_space: bool = True,
         characters: Optional[str] = None,
         files: bool = True,
         hide_hyperlink_hints: bool = False,
+        relative_dir: Optional[Path] = None,
     ) -> None:
         """Constructor."""
+        relative_dir = relative_dir if relative_dir is not None else pathlib.Path()
         self.elements["code_block"] = CustomCodeBlock
         self.elements["heading"] = CustomHeading
         self.elements["block_quote"] = CustomBlockQuote
@@ -290,6 +302,7 @@ class MarkdownOverwrite(markdown.Markdown):
         CustomImageItem.characters = characters
         CustomImageItem.files = files
         CustomImageItem.hide_hyperlink_hints = hide_hyperlink_hints
+        CustomImageItem.relative_dir = relative_dir
         super().__init__(
             markup=markup,
             code_theme=code_theme,
@@ -307,11 +320,12 @@ class CustomMarkdown:
 
     source: str
     theme: str
+    relative_dir: Path
     hyperlinks: bool = True
     nerd_font: bool = False
     unicode: bool = True
     images: bool = True
-    image_drawing: Literal["block", "character", "braille"] = "block"
+    image_drawing: ImageDrawing = "block"
     color: bool = True
     negative_space: bool = True
     characters: Optional[str] = None
@@ -337,6 +351,7 @@ class CustomMarkdown:
                 characters=self.characters,
                 files=self.files,
                 hide_hyperlink_hints=self.hide_hyperlink_hints,
+                relative_dir=self.relative_dir,
             )
         ]
 
@@ -364,11 +379,12 @@ def _splice_tables(
     nerd_font: bool,
     unicode: bool,
     images: bool,
-    image_drawing: Literal["block", "character", "braille"],
+    image_drawing: ImageDrawing,
     color: bool,
     negative_space: bool,
     files: bool,
     hide_hyperlink_hints: bool,
+    relative_dir: Path,
     characters: Optional[str] = None,
 ) -> Iterator[Union[MarkdownOverwrite, Table, Text]]:
     """Mix in tables with traditional markdown parser."""
@@ -391,6 +407,7 @@ def _splice_tables(
             characters=characters,
             files=files,
             hide_hyperlink_hints=hide_hyperlink_hints,
+            relative_dir=relative_dir,
         )
         yield text.Text()
         yield table_section.table
@@ -410,4 +427,5 @@ def _splice_tables(
         characters=characters,
         files=files,
         hide_hyperlink_hints=hide_hyperlink_hints,
+        relative_dir=relative_dir,
     )
